"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizePropsUsage = void 0;
const t = __importStar(require("@babel/types"));
const constants_1 = require("../constants");
/**
 * Recursively restructures an object chain from destructured param
 * Starts from the lowest end of the chain
 *
 * @param path Current NodePath in the chain
 * @param currentChain String array of chained properties
 */
const buildObjectChain = (path, currentChain = []) => {
    if ((path === null || path === void 0 ? void 0 : path.listKey) !== 'params' && (path === null || path === void 0 ? void 0 : path.parentPath)) {
        const { parentPath } = path;
        const { node } = parentPath;
        if (t.isObjectProperty(node) && t.isIdentifier(node.key)) {
            currentChain.unshift(node.key.name);
        }
        return buildObjectChain(parentPath.parentPath, currentChain);
    }
    currentChain.unshift(constants_1.PROPS_IDENTIFIER_NAME);
    return currentChain;
};
const arrowFunctionVisitor = {
    ArrowFunctionExpression(path) {
        var _a;
        const [propsParam] = path.get('params');
        if (propsParam) {
            const { node } = propsParam;
            if (t.isIdentifier(node) && node.name !== constants_1.PROPS_IDENTIFIER_NAME) {
                (_a = path.scope.getBinding(node.name)) === null || _a === void 0 ? void 0 : _a.referencePaths.forEach((reference) => {
                    reference.replaceWith(t.identifier(constants_1.PROPS_IDENTIFIER_NAME));
                });
                // If destructuring
            }
            else if (t.isObjectPattern(node)) {
                const { bindings } = path.scope;
                // @ts-expect-error
                // getBindingIdentifierPaths not in @babel/traverse types
                // But available since v6.20.0
                // https://github.com/babel/babel/pull/4876
                const destructedPaths = propsParam.getBindingIdentifierPaths();
                for (const key in destructedPaths) {
                    const binding = bindings[key];
                    if (binding.references) {
                        const objectChain = buildObjectChain(destructedPaths[key]);
                        binding.referencePaths.forEach((reference) => {
                            reference.replaceWithSourceString(objectChain.join('.'));
                        });
                    }
                }
            }
            propsParam.replaceWith(t.identifier(constants_1.PROPS_IDENTIFIER_NAME));
        }
    },
};
/**
 * Converts all usage of props to a normalized format
 * For example:
 *
 * `(p) => p.width`
 * `({ width }) => width`
 * `({ width: w }) => w`
 * `({height, ...rest}) => rest.width`
 *
 * All become
 * `(props) => props.width`
 *
 * @param styledPath Path of the CSS source
 */
const normalizePropsUsage = (styledPath) => {
    styledPath.traverse(arrowFunctionVisitor);
};
exports.normalizePropsUsage = normalizePropsUsage;
//# sourceMappingURL=normalize-props-usage.js.map